columns = [
    "report_date",
    "基本每股收益",
    "扣除非经常性损益每股收益",
    "每股未分配利润",
    "每股净资产",
    "每股资本公积金",
    "净资产收益率",
    "每股经营现金流量",
    "货币资金",
    "交易性金融资产",
    "应收票据",
    "应收账款",
    "预付款项",
    "其他应收款",
    "应收关联公司款",
    "应收利息",
    "应收股利",
    "存货",
    "其中：消耗性生物资产",
    "一年内到期的非流动资产",
    "其他流动资产",
    "流动资产合计",
    "可供出售金融资产",
    "持有至到期投资",
    "长期应收款",
    "长期股权投资",
    "投资性房地产",
    "固定资产",
    "在建工程",
    "工程物资",
    "固定资产清理",
    "生产性生物资产",
    "油气资产",
    "无形资产",
    "开发支出",
    "商誉",
    "长期待摊费用",
    "递延所得税资产",
    "其他非流动资产",
    "非流动资产合计",
    "资产总计",
    "短期借款",
    "交易性金融负债",
    "应付票据",
    "应付账款",
    "预收款项",
    "应付职工薪酬",
    "应交税费",
    "应付利息",
    "应付股利",
    "其他应付款",
    "应付关联公司款",
    "一年内到期的非流动负债",
    "其他流动负债",
    "流动负债合计",
    "长期借款",
    "应付债券",
    "长期应付款",
    "专项应付款",
    "预计负债",
    "递延所得税负债",
    "其他非流动负债",
    "非流动负债合计",
    "负债合计",
    "实收资本（或股本）",
    "资本公积",
    "盈余公积",
    "减：库存股",
    "未分配利润",
    "少数股东权益",
    "外币报表折算价差",
    "非正常经营项目收益调整",
    "所有者权益（或股东权益）合计",
    "负债和所有者（或股东权益）合计",
    "其中：营业收入",
    "其中：营业成本",
    "营业税金及附加",
    "销售费用",
    "管理费用",
    "勘探费用",
    "财务费用",
    "资产减值损失",
    "加：公允价值变动净收益",
    "投资收益",
    "其中：对联营企业和合营企业的投资收益",
    "影响营业利润的其他科目",
    "三、营业利润",
    "加：补贴收入",
    "营业外收入",
    "减：营业外支出",
    "其中：非流动资产处置净损失",
    "加：影响利润总额的其他科目",
    "四、利润总额",
    "减：所得税",
    "加：影响净利润的其他科目",
    "五、净利润",
    "归属于母公司所有者的净利润",
    "少数股东损益",
    "销售商品、提供劳务收到的现金",
    "收到的税费返还",
    "收到其他与经营活动有关的现金",
    "经营活动现金流入小计",
    "购买商品、接受劳务支付的现金",
    "支付给职工以及为职工支付的现金",
    "支付的各项税费",
    "支付其他与经营活动有关的现金",
    "经营活动现金流出小计",
    "经营活动产生的现金流量净额",
    "收回投资收到的现金",
    "取得投资收益收到的现金",
    "处置固定资产、无形资产和其他长期资产收回的现金净额",
    "处置子公司及其他营业单位收到的现金净额",
    "收到其他与投资活动有关的现金",
    "投资活动现金流入小计",
    "购建固定资产、无形资产和其他长期资产支付的现金",
    "投资支付的现金",
    "取得子公司及其他营业单位支付的现金净额",
    "支付其他与投资活动有关的现金",
    "投资活动现金流出小计",
    "投资活动产生的现金流量净额",
    "吸收投资收到的现金",
    "取得借款收到的现金",
    "收到其他与筹资活动有关的现金",
    "筹资活动现金流入小计",
    "偿还债务支付的现金",
    "分配股利、利润或偿付利息支付的现金",
    "支付其他与筹资活动有关的现金",
    "筹资活动现金流出小计",
    "筹资活动产生的现金流量净额",
    "四、汇率变动对现金的影响",
    "四(2)、其他原因对现金的影响",
    "五、现金及现金等价物净增加额",
    "期初现金及现金等价物余额",
    "期末现金及现金等价物余额",
    "净利润",
    "加：资产减值准备",
    "固定资产折旧、油气资产折耗、生产性生物资产折旧",
    "无形资产摊销",
    "长期待摊费用摊销",
    "处置固定资产、无形资产和其他长期资产的损失",
    "固定资产报废损失",
    "公允价值变动损失",
    "财务费用",
    "投资损失",
    "递延所得税资产减少",
    "递延所得税负债增加",
    "存货的减少",
    "经营性应收项目的减少",
    "经营性应付项目的增加",
    "其他",
    "经营活动产生的现金流量净额2",
    "债务转为资本",
    "一年内到期的可转换公司债券",
    "融资租入固定资产",
    "现金的期末余额",
    "减：现金的期初余额",
    "加：现金等价物的期末余额",
    "减：现金等价物的期初余额",
    "现金及现金等价物净增加额",
    "流动比率(非金融类指标)",
    "速动比率(非金融类指标)",
    "现金比率(%)(非金融类指标)",
    "利息保障倍数(非金融类指标)",
    "非流动负债比率(%)(非金融类指标)",
    "流动负债比率(%)(非金融类指标)",
    "现金到期债务比率(%)(非金融类指标)",
    "有形资产净值债务率(%)",
    "权益乘数(%)",
    "股东的权益/负债合计(%)",
    "有形资产/负债合计(%)",
    "经营活动产生的现金流量净额/负债合计(%)(非金融类指标)",
    "EBITDA/负债合计(%)(非金融类指标)",
    "应收帐款周转率(非金融类指标)",
    "存货周转率(非金融类指标)",
    "运营资金周转率(非金融类指标)",
    "总资产周转率(非金融类指标)",
    "固定资产周转率(非金融类指标)",
    "应收帐款周转天数(非金融类指标)",
    "存货周转天数(非金融类指标)",
    "流动资产周转率(非金融类指标)",
    "流动资产周转天数(非金融类指标)",
    "总资产周转天数(非金融类指标)",
    "股东权益周转率(非金融类指标)",
    "营业收入增长率(%)",
    "净利润增长率(%)",
    "净资产增长率(%)",
    "固定资产增长率(%)",
    "总资产增长率(%)",
    "投资收益增长率(%)",
    "营业利润增长率(%)",
    "扣非每股收益同比(%)",
    "扣非净利润同比(%)",
    "暂无",
    "成本费用利润率(%)",
    "营业利润率(非金融类指标)",
    "营业税金率(非金融类指标)",
    "营业成本率(非金融类指标)",
    "净资产收益率",
    "投资收益率",
    "销售净利率(%)",
    "总资产报酬率",
    "净利润率(非金融类指标)",
    "销售毛利率(%)(非金融类指标)",
    "三费比重(非金融类指标)",
    "管理费用率(非金融类指标)",
    "财务费用率(非金融类指标)",
    "扣除非经常性损益后的净利润",
    "息税前利润(EBIT)",
    "息税折旧摊销前利润(EBITDA)",
    "EBITDA/营业总收入(%)(非金融类指标)",
    "资产负债率(%)",
    "流动资产比率(非金融类指标)",
    "货币资金比率(非金融类指标)",
    "存货比率(非金融类指标)",
    "固定资产比率",
    "负债结构比(非金融类指标)",
    "归属于母公司股东权益/全部投入资本(%)",
    "股东的权益/带息债务(%)",
    "有形资产/净债务(%)",
    "每股经营性现金流(元)",
    "营业收入现金含量(%)(非金融类指标)",
    "经营活动产生的现金流量净额/经营活动净收益(%)",
    "销售商品提供劳务收到的现金/营业收入(%)",
    "经营活动产生的现金流量净额/营业收入",
    "资本支出/折旧和摊销",
    "每股现金流量净额(元)",
    "经营净现金比率（短期债务）(非金融类指标)",
    "经营净现金比率（全部债务）",
    "经营活动现金净流量与净利润比率",
    "全部资产现金回收率",
    "营业收入",
    "营业利润",
    "归属于母公司所有者的净利润",
    "扣除非经常性损益后的净利润",
    "经营活动产生的现金流量净额",
    "投资活动产生的现金流量净额",
    "筹资活动产生的现金流量净额",
    "现金及现金等价物净增加额",
    "总股本",
    "已上市流通A股",
    "已上市流通B股",
    "已上市流通H股",
    "股东人数(户)",
    "第一大股东的持股数量",
    "十大流通股东持股数量合计(股)",
    "十大股东持股数量合计(股)",
    "机构总量（家）",
    "机构持股总量(股)",
    "QFII机构数",
    "QFII持股量",
    "券商机构数",
    "券商持股量",
    "保险机构数",
    "保险持股量",
    "基金机构数",
    "基金持股量",
    "社保机构数",
    "社保持股量",
    "私募机构数",
    "私募持股量",
    "财务公司机构数",
    "财务公司持股量",
    "年金机构数",
    "年金持股量",
    "十大流通股东中持有A股合计(股)",
    "第一大流通股东持股量(股)",
    "自由流通股(股)",
    "受限流通A股(股)",
    "一般风险准备(金融类)",
    "其他综合收益(利润表)",
    "综合收益总额(利润表)",
    "归属于母公司股东权益(资产负债表)",
    "银行机构数(家)(机构持股)",
    "银行持股量(股)(机构持股)",
    "一般法人机构数(家)(机构持股)",
    "一般法人持股量(股)(机构持股)",
    "近一年净利润(元)",
    "信托机构数(家)(机构持股)",
    "信托持股量(股)(机构持股)",
    "特殊法人机构数(家)(机构持股)",
    "特殊法人持股量(股)(机构持股)",
    "加权净资产收益率(每股指标)",
    "扣非每股收益(单季度财务指标)",
    "最近一年营业收入（万元）",
    "国家队持股数量（万股)",
    "业绩预告-本期净利润同比增幅下限%",
    "业绩预告-本期净利润同比增幅上限%",
    "归母净利润（业绩快报）",
    "扣非净利润（业绩快报）",
    "总资产（业绩快报）",
    "净资产（业绩快报）",
    "每股收益（业绩快报）",
    "摊薄净资产收益率（业绩快报）",
    "加权净资产收益率（业绩快报）",
    "每股净资产（业绩快报）",
    "应付票据及应付账款(资产负债表)",
    "应收票据及应收账款(资产负债表)",
    "递延收益(资产负债表)",
    "其他综合收益(资产负债表)",
    "其他权益工具(资产负债表)",
    "其他收益(利润表)",
    "资产处置收益(利润表)",
    "持续经营净利润(利润表)",
    "终止经营净利润(利润表)",
    "研发费用(利润表)",
    "其中:利息费用(利润表-财务费用)",
    "其中:利息收入(利润表-财务费用)",
    "近一年经营活动现金流净额",
    "近一年归母净利润（万元）",
    "近一年扣非净利润（万元）",
    "近一年现金净流量（万元）",
    "基本每股收益（单季度）",
    "营业总收入(单季度)(万元)",
    "业绩预告公告日期 ",
    "财报公告日期",
    "业绩快报公告日期",
    "近一年投资活动现金流净额(万元)",
    "业绩预告-本期净利润下限(万元)",
    "业绩预告-本期净利润上限(万元)",
    "营业总收入TTM(万元)",
    "员工总数(人)",
    "每股企业自由现金流",
    "每股股东自由现金流",
    "col323",
    "col324",
    "col325",
    "col326",
    "col327",
    "col328",
    "col329",
    "col330",
    "col331",
    "col332",
    "col333",
    "col334",
    "col335",
    "col336",
    "col337",
    "col338",
    "col339",
    "col340",
    "col341",
    "col342",
    "col343",
    "col344",
    "col345",
    "col346",
    "col347",
    "col348",
    "col349",
    "col350",
    "col351",
    "col352",
    "col353",
    "col354",
    "col355",
    "col356",
    "col357",
    "col358",
    "col359",
    "col360",
    "col361",
    "col362",
    "col363",
    "col364",
    "col365",
    "col366",
    "col367",
    "col368",
    "col369",
    "col370",
    "col371",
    "col372",
    "col373",
    "col374",
    "col375",
    "col376",
    "col377",
    "col378",
    "col379",
    "col380",
    "col381",
    "col382",
    "col383",
    "col384",
    "col385",
    "col386",
    "col387",
    "col388",
    "col389",
    "col390",
    "col391",
    "col392",
    "col393",
    "col394",
    "col395",
    "col396",
    "col397",
    "col398",
    "col399",
    "col400",
    "专项储备(万元)",
    "结算备付金(万元)",
    "拆出资金(万元)",
    "发放贷款及垫款(万元)(流动资产科目)",
    "衍生金融资产(万元)",
    "应收保费(万元)",
    "应收分保账款(万元)",
    "应收分保合同准备金(万元)",
    "买入返售金融资产(万元)",
    "划分为持有待售的资产(万元)",
    "发放贷款及垫款(万元)(非流动资产科目)",
    "向中央银行借款(万元)",
    "吸收存款及同业存放(万元)",
    "拆入资金(万元)",
    "衍生金融负债(万元)",
    "卖出回购金融资产款(万元)",
    "应付手续费及佣金(万元)",
    "应付分保账款(万元)",
    "保险合同准备金(万元)",
    "代理买卖证券款(万元)",
    "代理承销证券款(万元)",
    "划分为持有待售的负债(万元)",
    "预计负债(万元)",
    "递延收益(万元)（流动负债科目）",
    "其中:优先股(万元)(非流动负债科目)",
    "永续债(万元)(非流动负债科目)",
    "长期应付职工薪酬(万元)",
    "其中:优先股(万元)(所有者权益科目)",
    "永续债(万元)(所有者权益科目)",
    "债权投资(万元)",
    "其他债权投资(万元)",
    "其他权益工具投资(万元)",
    "其他非流动金融资产(万元)",
    "合同负债(万元)",
    "合同资产(万元)",
    "其他资产(万元)",
    "应收款项融资(万元)",
    "使用权资产(万元)",
    "租赁负债(万元)",
    "col440",
    "col441",
    "col442",
    "col443",
    "col444",
    "col445",
    "col446",
    "col447",
    "col448",
    "col449",
    "col450",
    "col451",
    "col452",
    "col453",
    "col454",
    "col455",
    "col456",
    "col457",
    "col458",
    "col459",
    "col460",
    "col461",
    "col462",
    "col463",
    "col464",
    "col465",
    "col466",
    "col467",
    "col468",
    "col469",
    "col470",
    "col471",
    "col472",
    "col473",
    "col474",
    "col475",
    "col476",
    "col477",
    "col478",
    "col479",
    "col480",
    "col481",
    "col482",
    "col483",
    "col484",
    "col485",
    "col486",
    "col487",
    "col488",
    "col489",
    "col490",
    "col491",
    "col492",
    "col493",
    "col494",
    "col495",
    "col496",
    "col497",
    "col498",
    "col499",
    "col500",
    "稀释每股收益(元)",
    "营业总收入(万元)",
    "汇兑收益(万元)",
    "其中:归属于母公司综合收益(万元)",
    "其中:归属于少数股东综合收益(万元)",
    "利息收入(万元)",
    "已赚保费(万元)",
    "手续费及佣金收入(万元)",
    "利息支出(万元)",
    "手续费及佣金支出(万元)",
    "退保金(万元)",
    "赔付支出净额(万元)",
    "提取保险合同准备金净额(万元)",
    "保单红利支出(万元)",
    "分保费用(万元)",
    "其中:非流动资产处置利得(万元)",
    "信用减值损失(万元)",
    "净敞口套期收益(万元)",
    "营业总成本(万元)",
    "信用减值损失(万元、2019格式)",
    "资产减值损失(万元、2019格式)",
    "col522",
    "col523",
    "col524",
    "col525",
    "col526",
    "col527",
    "col528",
    "col529",
    "col530",
    "col531",
    "col532",
    "col533",
    "col534",
    "col535",
    "col536",
    "col537",
    "col538",
    "col539",
    "col540",
    "col541",
    "col542",
    "col543",
    "col544",
    "col545",
    "col546",
    "col547",
    "col548",
    "col549",
    "col550",
    "col551",
    "col552",
    "col553",
    "col554",
    "col555",
    "col556",
    "col557",
    "col558",
    "col559",
    "col560",
    "加:其他原因对现金的影响2(万元)(现金的期末余额科目)",
    "客户存款和同业存放款项净增加额(万元)",
    "向中央银行借款净增加额(万元)",
    "向其他金融机构拆入资金净增加额(万元)",
    "收到原保险合同保费取得的现金(万元)",
    "收到再保险业务现金净额(万元)",
    "保户储金及投资款净增加额(万元)",
    "处置以公允价值计量且其变动计入当期损益的金融资产净增加额(万元)",
    "收取利息、手续费及佣金的现金(万元)",
    "拆入资金净增加额(万元)",
    "回购业务资金净增加额(万元)",
    "客户贷款及垫款净增加额(万元)",
    "存放中央银行和同业款项净增加额(万元)",
    "支付原保险合同赔付款项的现金(万元)",
    "支付利息、手续费及佣金的现金(万元)",
    "支付保单红利的现金(万元)",
    "其中:子公司吸收少数股东投资收到的现金(万元)",
    "其中:子公司支付给少数股东的股利、利润(万元)",
    "投资性房地产的折旧及摊销(万元)",
    "信用减值损失(万元)",
]



# # @Author  : BoPo
# # @Time    : 2022/1/18 15:47
# # @Function:
# # array(['year', 'month', 'day', 'category', 'name', 'fenhong', 'peigujia',
# #        'songzhuangu', 'peigu', 'suogu', 'panqianliutong', 'panhouliutong',
# #        'qianzongguben', 'houzongguben', 'fenshu', 'xingquanjia'],
# #       dtype=object)
# # present  bonus  price  rationed
# # 展示      奖金    价格    配给
# # category == 1
# # fenhong 分红
# # peigujia 配股价
# # songzhuangu 送转股
# # peigu 配股
# # suogu 缩股
# # panqianliutong  盘前流通
# # qianzongguben 前总股本
# # houzongguben 后总股本
# # fenshu 分属
# # xingquanjia 行权价
# # http://www.allparty.cn/yong-Pandas-ji-suan-qian-fu-quan-shu-ju.html
# # http://www.cppblog.com/wangkang2009/archive/2015/04/25/210437.aspx
# # https://raw.githubusercontent.com/yutiansut/QUANTAXIS/master/QUANTAXIS/QAData/data_fq.py
# # https://www.cnblogs.com/whiteBear/p/12782161.html
# # https://www.cnblogs.com/weidu/p/9831807.html
# import pandas as pd
#
# from mootdx.quotes import Quotes
#
#
# def calc_fuquan_use_fenhong(df, df_fenhong):
#     """获取复权后的历史数据, 用分红表来计算复权 , 前复权
#     df: 日k线
#     df_fenhong: 分红表
#     return: df"""
#     # 日期早的在前面
#     # 经过测试， 前复权结果与同花顺，通达信的计算相同
#
#     df_fenhong = df_fenhong.sort_index(by=2)
#
#     for i in range(len(df_fenhong)):
#         gu, money, date = df_fenhong.irow(i)
#
#         if len(df.ix[:date]) < 2:
#             continue
#
#         # 比对日期
#         date = agl.df_get_pre_date(df, date)
#
#         if money > 0:
#             money = money * 0.1
#             df["o"].ix[:date] -= money
#             df["h"].ix[:date] -= money
#             df["c"].ix[:date] -= money
#             df["l"].ix[:date] -= money
#         if gu > 0:
#             # x = cur / (1+y/10)
#             gu = 1 + gu / 10
#             df["o"].ix[:date] /= gu
#             df["h"].ix[:date] /= gu
#             df["c"].ix[:date] /= gu
#             df["l"].ix[:date] /= gu
#
#     return df
#
#
# def make_fq(symbol="", method="00"):
#     client = Quotes.factory(market="std")
#     data = client.bars(symbol=symbol)
#     xdxr = client.xdxr(symbol=symbol).query("category==1")
#
#     if method:
#         return make_qfq(data, xdxr, method)
#
#     return data
#
#
# # present  bonus  price  rationed
# # 展示      奖金    价格    配给
#
#
# def make_qfq(data, xdxr, fq_type="01"):
#     """使用数据库数据进行复权"""
#
#     # 过滤其他，只留除权信息
#     xdxr = xdxr.query("category==1")
#     # data = data.assign(if_trade=1)
#
#     if len(xdxr) > 0:
#         # 有除权信息, 合并原数据 + 除权数据
#         # data = pd.concat([data, xdxr.loc[data.index[0]:data.index[-1], ['category']]], axis=1)
#         # data['if_trade'].fillna(value=0, inplace=True)
#
#         data = data.fillna(method="ffill")
#         # present       bonus       price       rationed
#         # songzhuangu   fenhong     peigujia    peigu
#         data = pd.concat(
#             [data, xdxr.loc[data.index[0] : data.index[-1], ["fenhong", "peigu", "peigujia", "songzhuangu"]]], axis=1
#         )
#     else:
#         # 没有除权信息
#         data = pd.concat([data, xdxr.loc[:, ["fenhong", "peigu", "peigujia", "songzhuangu"]]], axis=1)
#
#     # 清理数据
#     data = data.fillna(0)
#
#     if fq_type == "01":
#         # 生成 preclose todo 关键位置
#
#         # for key,val in ea.iterrows():
#         #     date = key - datetime.timedelta(days=1)
#         #     for field in df.columns.values:
#         #         if field != 'volume' and field != 'amount':
#         #             df.iloc[date:, field] -= val.bonus/10
#         #             df.iloc[date:, field] += val.price*(val.rationed/10)
#         #             df.iloc[date:, field] /= 1 + val.present/10 + val.rationed/10
#
#         # -= val.bonus / 10 == @todo
#         df1 = data["close"].shift(1) * 10 - data["fenhong"] + data["peigu"] * data["peigujia"]
#         df2 = 10 + data["peigu"] + data["songzhuangu"]
#
#         data["preclose"] = (data["close"].shift(1) * 10 - data["fenhong"] + data["peigu"] * data["peigujia"]) / (
#             10 + data["peigu"] + data["songzhuangu"]
#         )
#         # 生成 adj 复权因子
#         data["adj"] = (data["preclose"].shift(-1) / data["close"]).fillna(1)[::-1].cumprod()
#     else:
#         # 生成 preclose todo 关键位置
#         data["preclose"] = (data["close"].shift(1) * 10 - data["fenhong"] + data["peigu"] * data["peigujia"]) / (
#             10 + data["peigu"] + data["songzhuangu"]
#         )
#         # 生成 adj 复权因子
#         data["adj"] = (data["close"] / data["preclose"].shift(-1)).cumprod().shift(1).fillna(1)
#
#     # 计算复权价格
#     for field in data.columns.values:
#         if field in ("open", "close", "high", "low", "preclose"):
#             data[field] = data[field] * data["adj"]
#
#     # 清理数据, 返回结果
#     return data.query("open != 0").drop(
#         [
#             "fenhong",
#             "peigu",
#             "peigujia",
#             "songzhuangu",
#         ],
#         axis=1,
#     )
#
#
# # def make_hfq(bfq_data, xdxr_data):
# #     """使用数据库数据进行复权"""
# #     info = xdxr_data.query('category==1')
# #     bfq_data = bfq_data.assign(if_trade=1)
# #
# #     if len(info) > 0:
# #         # 合并数据
# #         data = pd.concat([bfq_data, info.loc[bfq_data.index[0]:bfq_data.index[-1], ['category']]], axis=1)
# #         data['if_trade'].fillna(value=0, inplace=True)
# #
# #         data = data.fillna(method='ffill')
# #         data = pd.concat([data, info.loc[bfq_data.index[0]:bfq_data.index[-1], ['fenhong', 'peigu', 'peigujia', 'songzhuangu']]], axis=1)
# #     else:
# #         data = pd.concat([bfq_data, info.loc[:, ['category', 'fenhong', 'peigu', 'peigujia', 'songzhuangu']]], axis=1)
# #
# #     data = data.fillna(0)
# #
# #     # 生成 preclose todo 关键位置
# #     data['preclose'] = (data['close'].shift(1) * 10 - data['fenhong'] + data['peigu'] * data['peigujia']) / (10 + data['peigu'] + data['songzhuangu'])
# #     data['adj'] = (data['close'] / data['preclose'].shift(-1)).cumprod().shift(1).fillna(1)
# #
# #     # 计算复权价格
# #     for field in data.columns.values:
# #         if field in ('open', 'close', 'high', 'low', 'preclose'):
# #             data[field] = data[field] * data['adj']
# #
# #     # data['open'] = data['open'] * data['adj']
# #     # data['high'] = data['high'] * data['adj']
# #     # data['low'] = data['low'] * data['adj']
# #     # data['close'] = data['close'] * data['adj']
# #     # data['preclose'] = data['preclose'] * data['adj']
# #
# #     # 不计算 交易量
# #     # data['volume'] = data['volume'] / data['adj'] if 'volume' in data.columns else data['vol'] / data['adj']
# #
# #     try:
# #         data['high_limit'] = data['high_limit'] * data['adj']
# #         data['low_limit'] = data['high_limit'] * data['adj']
# #     except:
# #         pass
# #
# #     return data.query('if_trade==1 and open != 0').drop(['fenhong', 'peigu', 'peigujia', 'songzhuangu', 'if_trade', 'category'], axis=1)



import logging
import os
from pathlib import Path

import click
from prettytable import PrettyTable

from mootdx import __version__
from mootdx.logger import logger
from mootdx.utils import get_config_path
from mootdx.utils import to_file


# @click.group()
# @click.option('--debug/--no-debug', default=False)
# @click.pass_context
# def cli(ctx, debug):
#     ctx.obj['DEBUG'] = debug
#     ctx.obj['DEBUG'] and logging.basicConfig(level=logging.DEBUG)


@click.group()
@click.version_option(__version__, "-V", "--version", prog_name="Mootdx", message="%(prog)s: v%(version)s")
@click.help_option("-h", "--help")
def entry():
    ...


@entry.command(help="读取股票在线行情数据.")
@click.help_option("-h", "--help")
@click.option("-o", "--output", default=None, help="输出文件, 支持CSV, HDF5, Excel等格式.")
@click.option("-s", "--symbol", default="600000", help="股票代码.")
@click.option("-a", "--action", default="bars", help="操作类型 (daily: 日线, minute: 一分钟线, fzline: 五分钟线).", )
@click.option("-m", "--market", default="std", help="证券市场, 默认 std (std: 标准股票市场, ext: 扩展市场).")
def quotes(symbol, action, market, output):
    from mootdx.quotes import Quotes

    client = Quotes.factory(market=market, multithread=True)

    try:
        action = "bars" if "daily" else action
        if action == "daily":
            frequency = 9
        elif action == "minute":
            frequency = 8
        elif action == "fzline":
            frequency = 0
        else:
            frequency = 9

        feed = getattr(client, "bars")(symbol=symbol, frequency=frequency)
        to_file(feed, output) if output else None
        click.echo(feed)
    except Exception as e:
        raise e


@entry.command(help="读取股票本地行情数据.")
@click.help_option("-h", "--help")
@click.option("-d", "--tdxdir", default="C:/new_tdx", help="通达信数据目录.")
@click.option("-s", "--symbol", default="600000", help="股票代码.")
@click.option("-a", "--action", default="daily", help="操作类型 (daily: 日线, minute: 一分钟线, fzline: 五分钟线).")
@click.option("-m", "--market", default="std", help="证券市场, 默认 std (std: 标准股票市场, ext: 扩展市场).")
@click.option("-o", "--output", default=None, help="输出文件, 支持 CSV, HDF5, Excel 等格式.")
def reader(symbol, action, market, tdxdir, output):
    from mootdx.reader import Reader

    client = Reader.factory(market=market, tdxdir=tdxdir)

    try:
        feed = getattr(client, action)(symbol=symbol)
        to_file(feed, output) if output else None
        click.echo(feed)
    except Exception as e:
        raise e


@entry.command(help="添加行情服务器.", name="remote")
@click.help_option("-h", "--help")
@click.option("-v", "--verbose", count=True, help="详细模式")
@click.argument("server")
def channel(server, verbose):
    config = get_config_path("config.json")
    logger.info("[√] 已经将最优服务器IP写入配置文件 {} {}".format(config, server))


@entry.command(help="测试行情服务器.", name="bestip")
@click.help_option("-h", "--help")
@click.option("-l", "--limit", default=5, help="显示最快前几个，默认 5.")
@click.option("-v", "--verbose", count=True, help="详细模式")
def server(limit, verbose):
    from mootdx.server import bestip

    if verbose:
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)

        logger.addHandler(ch)
        logger.setLevel(logging.DEBUG)

    bestip(limit=limit, console=True, sync=False)

    config = get_config_path("config.json")
    logger.info("[√] 已经将最优服务器IP写入配置文件 {}".format(config))


@entry.command(help="财务文件下载&解析.")
@click.help_option("-h", "--help")
@click.option("-p", "--parse", default=None, help="要解析文件名")
@click.option("-f", "--fetch", default=None, help="下载财务文件的文件名")
@click.option("-a", "--downall", is_flag=True, help="下载全部文件")
@click.option("-o", "--output", default=None, help="输出文件, 支持 CSV, HDF5, Excel, JSON 等格式.")
@click.option("-d", "--downdir", default="output", help="下载文件目录")
@click.option("-l", "--listfile", is_flag=True, default=False, help="显示全部文件")
@click.option("-v", "--verbose", count=True, help="详细模式")
def affair(parse, fetch, downdir, output, downall, verbose, listfile):
    from mootdx.affair import Affair

    if verbose:
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)

        logger.addHandler(ch)
        logger.setLevel(logging.DEBUG)

    files = Affair.files()

    if listfile:
        t = PrettyTable(["filename", "filesize", "hash"])
        t.align["filename"] = "l"
        t.align["filesize"] = "l"
        t.align["hash"] = "l"
        t.padding_width = 0

        for x in files:
            t.add_row([x["filename"], x["filesize"], x["hash"]])

        click.echo(t)
        return

    # download all files
    if downall or fetch == "all":
        feed = Affair.fetch(downdir=downdir)
        return to_file(feed, output) if output else None

    # download file
    if fetch:
        return Affair.fetch(downdir=downdir, filename=fetch.strip(".zip") + ".zip")

    # parse file
    if parse:
        parse = parse.strip(".zip") + ".zip"
        files = [x["filename"] for x in files]

        if parse in files:
            # 文件不存在，先下载
            Path(downdir, parse).exists() or Affair.fetch(downdir=downdir, filename=parse)
            feed = Affair.parse(downdir=downdir, filename=parse)
            output and to_file(feed, output)
            click.echo(feed)
        else:
            logger.error("没找到要解析的文件.")


@entry.command(help="批量下载行情数据.")
@click.help_option("-h", "--help")
@click.option("-o", "--output", default="bundle", help="转存文件目录.")
@click.option("-s", "--symbol", default="600000", help="股票代码. 多个用,隔开")
@click.option("-a", "--action", default="bars", help="操作类型 (daily: 日线, minute: 一分钟线, fzline: 五分钟线).")
@click.option("-m", "--market", default="std", help="证券市场, 默认 std (std: 标准股票市场, ext: 扩展市场).")
@click.option("-e", "--extension", default="csv", help="转存文件的格式, 支持 CSV, HDF5, Excel, JSON 等格式.")
def bundle(symbol, action, market, output, extension):
    """
    批量下载行情数据
    :return:
    """
    from mootdx.quotes import Quotes

    client = Quotes.factory(market=market, multithread=True)
    symbol = symbol.replace("，", ",").strip(",").split(",")

    for code in symbol:
        try:
            if action == "daily":
                frequency = 9
            elif action == "minute":
                frequency = 8
            elif action == "fzline":
                frequency = 0
            else:
                frequency = 9

            feed = getattr(client, "bars")(symbol=code, frequency=frequency)
            output and to_file(feed, os.path.join(output, f"{code}.{extension}"))
            click.echo("下载完成 {}".format(code))
        except Exception as e:
            raise e

    click.echo('[√] 下载文件到 "{}"'.format(os.path.realpath(output)))


if __name__ == "__main__":
    entry()



import pandas as pd


def _reversion(bfq_data, xdxr_data, type_):
    """使用数据库数据进行复权"""
    info = xdxr_data.query("category==1")
    bfq_data = bfq_data.assign(if_trade=1)

    if len(info) > 0:
        # 有除权数据
        data = pd.concat([bfq_data, info.loc[bfq_data.index[0]: bfq_data.index[-1], ["category"]]], axis=1)
        data["if_trade"].fillna(value=0, inplace=True)
        data = data.fillna(method="ffill")
        data = pd.concat([data, info.loc[bfq_data.index[0]: bfq_data.index[-1], ["fenhong", "peigu", "peigujia", "songzhuangu"]]], axis=1)
    else:
        data = pd.concat([bfq_data, info.loc[:, ["category", "fenhong", "peigu", "peigujia", "songzhuangu"]]], axis=1)

    # 数据补全
    data = data.fillna(0)

    # 计算前日收盘
    data["preclose"] = (data["close"].shift(1) * 10 - data["fenhong"] + data["peigu"] * data["peigujia"]) / (10 + data["peigu"] + data["songzhuangu"])

    # 前复权
    if type_ in ["01", "qfq"]:
        data["adj"] = (data["preclose"].shift(-1) / data["close"]).fillna(1)[::-1].cumprod()

    # 后复权
    if type_ in ["02", "hfq"]:
        data["adj"] = (data["close"] / data["preclose"].shift(-1)).cumprod().shift(1).fillna(1)

    # ohlc 数据进行复权计算
    for col in ["open", "high", "low", "close", "preclose"]:
        data[col] = data[col] * data["adj"]

    data["volume"] = data.get("volume", data.get("vol"))
    data['volume'] = data['volume'] / data['adj']
    # data['volume'] = data['volume'] / data['adj'] if 'volume' in data.columns else data['vol'] / data['adj']

    try:
        # 大该是涨跌幅
        data["high_limit"] = data["high_limit"] * data["adj"]
        data["low_limit"] = data["low_limit"] * data["adj"]
    except:
        pass

    return data.query("if_trade==1 and open != 0").drop(["fenhong", "peigu", "peigujia", "songzhuangu", "if_trade", "category"], axis=1, errors="ignore")


def reversion(stock_data, xdxr, type_="01"):
    def _fetch_xdxr(symbol, collections=None):
        """获取股票除权信息/数据库"""
        columns = [
            "category",
            "category_meaning",
            "code",
            "date",
            "fenhong",
            "fenshu",
            "liquidity_after",
            "liquidity_before",
            "name",
            "peigu",
            "peigujia",
            "shares_after",
            "shares_before",
            "songzhuangu",
            "suogu",
            "xingquanjia",
        ]

        try:
            data = pd.DataFrame([item for item in collections.find({"code": symbol})]).drop(["_id"], axis=1)
            data["date"] = pd.to_datetime(data["date"], utc=False)
            return data.set_index(["date", "code"], drop=False)
        except:
            return pd.DataFrame(data=[], columns=columns)

    # '股票 日线/分钟线 动态复权接口'
    symbol = (
        stock_data.index.remove_unused_levels().levels[1][0]
        if isinstance(stock_data.index, pd.MultiIndex)
        else stock_data["code"][0]
    )
    return _reversion(bfq_data=stock_data, xdxr_data=_fetch_xdxr(symbol, xdxr), type_=type_)


# 算法一样
def baoli_qfq(df, xdxr):
    fenhong = xdxr["fenhong"]  # 分红
    peigu = xdxr["peigu"]  # 配股
    peigujia = xdxr["peigujia"]  # 配股价
    songzhuangu = xdxr["songzhuangu"]  # 送转股

    for i in range(0, len(xdxr)):
        fh = fenhong[i]
        pg = peigu[i]
        pgj = peigujia[i]
        szg = songzhuangu[i]
        date = xdxr.index[i]

        df.loc[df.index < date, "open"] = (df["open"][df.index < date] * 10 - fh + pg * pgj) / (10 + pg + szg)
        df.loc[df.index < date, "close"] = (df["close"][df.index < date] * 10 - fh + pg * pgj) / (10 + pg + szg)
        df.loc[df.index < date, "high"] = (df["high"][df.index < date] * 10 - fh + pg * pgj) / (10 + pg + szg)
        df.loc[df.index < date, "low"] = (df["low"][df.index < date] * 10 - fh + pg * pgj) / (10 + pg + szg)

    return df



import asyncio
import functools
import json
import socket
import time
from functools import partial

from tdxpy.constants import hq_hosts
from tdxpy.exhq import TdxExHq_API
from tdxpy.hq import TdxHq_API

from mootdx.consts import CONFIG
from mootdx.consts import EX_HOSTS
from mootdx.consts import GP_HOSTS
from mootdx.logger import logger
from mootdx.utils import get_config_path

hosts = {
    "HQ": [{"addr": hs[1], "port": hs[2], "time": 0, "site": hs[0]} for hs in hq_hosts + GP_HOSTS],
    "EX": [{"addr": hs[1], "port": hs[2], "time": 0, "site": hs[0]} for hs in EX_HOSTS],
    "GP": [{"addr": hs[1], "port": hs[2], "time": 0, "site": hs[0]} for hs in GP_HOSTS],
}

results = {k: [] for k in hosts}


def callback(res, key):
    """
    异步回调函数

    :param res:
    :param key:
    """
    result = res.result()

    if result.get("time"):
        results[key].append(result)

    # logger.debug(f"callback: {res.result()}")


def connect(proxy: dict) -> dict:
    """
    连接服务器函数

    :param proxy: 代理IP信息
    :return:
    """
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(3)

        start = time.perf_counter()

        sock.connect((proxy.get("addr"), int(proxy.get("port"))))
        sock.close()

        proxy["time"] = (time.perf_counter() - start) * 1000

        logger.debug("{addr}:{port} 验证通过，响应时间：{time} ms.".format(**proxy))
    except socket.timeout as ex:
        logger.debug("{addr},{port} time out.".format(**proxy))
        proxy["time"] = None
    except ConnectionRefusedError as ex:
        logger.debug("{addr},{port} 验证失败.".format(**proxy))
        proxy["time"] = None

    return proxy


def connect2(proxy, index='HQ'):
    if index == 'GP':
        return connect(proxy)

    api = (TdxHq_API(), TdxExHq_API())[index != 'HQ']
    fun = ('get_security_count', 'get_instrument_count')[index != 'HQ']
    tms = time.perf_counter()

    proxy["time"] = None

    try:
        with api.connect(proxy.get("addr"), int(proxy.get("port")), time_out=0.7):
            if getattr(api, fun)():
                proxy["time"] = (time.perf_counter() - tms) * 1000
                logger.debug("{addr}:{port} 验证通过，响应时间：{time} ms.".format(**proxy))
            else:
                logger.debug("{addr},{port} 验证失败.".format(**proxy))
    except socket.timeout as ex:
        logger.debug("{addr},{port} time out.".format(**proxy))
        proxy["time"] = None
    except Exception:
        logger.debug("{addr},{port} 验证失败.".format(**proxy))
    finally:
        logger.debug("{addr},{port} connect2.".format(**proxy))

    return proxy


async def verify(proxy: dict, index):
    """
    检验代理连通性函数

    :param proxy: 代理IP信息
    :return:
    """
    return await asyncio.get_event_loop().run_in_executor(None, functools.partial(connect2, proxy=proxy, index=index))


def server(index=None, limit=5, console=False, sync=True):
    _hosts = hosts[index]

    def async_event():
        event = asyncio.get_event_loop()
        tasks = []

        while len(_hosts) > 0:
            task = event.create_task(verify(_hosts.pop(0), index))
            task.add_done_callback(partial(callback, key=index))
            tasks.append(task)

        # event.is_closed()
        # event.is_running()
        event.run_until_complete(asyncio.wait(tasks))

    global results

    if sync:
        results[index] = [connect(proxy) for proxy in _hosts]
        results[index] = [x for x in results[index] if x.get("time")]
    else:
        async_event()

    server = results[index]

    # 结果按响应时间从小到大排序
    if console:
        from prettytable import PrettyTable

        server.sort(key=lambda item: item["time"])

        if limit:
            server = server[:limit]

        logger.debug("[√] 最优服务器:")

        t = PrettyTable(["Name", "Addr", "Port", "Time"])
        t.align["Name"] = "l"
        t.align["Addr"] = "l"
        t.align["Port"] = "l"
        t.align["Time"] = "r"
        t.padding_width = 1

        for host in server:
            t.add_row(
                [
                    host["site"],
                    host["addr"],
                    host["port"],
                    "{:5.2f} ms".format(host["time"]),
                ]
            )

        logger.debug("\n" + str(t))

    return [(item["addr"], item["port"]) for item in server]


def check_server(console=False, limit=5, sync=False) -> None:
    return bestip(console=console, limit=limit, sync=sync)


def bestip(console=False, limit=5, sync=False) -> None:
    config_ = get_config_path("config.json")
    default = dict(CONFIG)

    logger.info("[-] 选择最快的服务器...")
    logger.debug(f'sync => {sync}')

    for index in ["HQ", "EX", "GP"]:
        try:
            data = server(index=index, limit=limit, console=console, sync=sync)

            if data:
                default["BESTIP"][index] = data[0]
        except RuntimeError as ex:
            logger.error("请手动运行`python -m mootdx bestip`")
            break

    json.dump(default, open(config_, "w", encoding="utf-8"), indent=2, ensure_ascii=False)


if __name__ == "__main__":
    bestip(sync=False, limit=5, console=True)
